
#
# from kynex.llm.gemini.gemini import GeminiLLM
# from kynex.llm.groq.groq import GroqLLM
# from kynex.llm.ollama.ollama import OllamaLLMWrapper as OllamaLLM
#
# class LLMConnector:
#     def __init__(self, model_name: str, llm_type: str, host: str = None):
#         self.model_name = model_name
#         self.llm_type = llm_type.lower()
#         self.host = host  # Host required for Ollama
#
#     def get_llm_instance(self):
#         print(f"🔍 Selected LLM: {self.llm_type} | Model: {self.model_name}")
#
#         if self.llm_type == "gemini":
#             return GeminiLLM(model_name=self.model_name)
#         elif self.llm_type == "groq":
#             return GroqLLM(model_name=self.model_name)
#         elif self.llm_type == "ollama":
#             if not self.host:
#                 raise ValueError("Host is required for Ollama LLM.")
#             return OllamaLLM(model_name=self.model_name, host=self.host)
#         else:
#             raise ValueError(f"Unsupported LLM type: {self.llm_type}")
#
#     def getLLMData(self, prompt: str) -> str:
#         llm = self.get_llm_instance()
#         return llm.get_data(prompt)

#
# from kynex.llm.gemini.GeminiLLMWrapper import GeminiLLMWrapper
# from kynex.llm.groq.GroqLLMWrapper import GroqLLMWrapper
# from kynex.llm.ollama.OllamaLLMWrapper import OllamaLLMWrapper
#
# class LLMConnectorHelper:
#     def __init__(self, model_name: str, llm_type: str, api_key: str = None, host: str = None):
#         self.model_name = model_name
#         self.llm_type = llm_type.lower()
#         self.api_key = api_key
#         self.host = host
#
#     def get_llm_instance(self):
#         print(f" Selected LLM: {self.llm_type} | Model: {self.model_name}")
#
#         if self.llm_type == "gemini":
#             return GeminiLLMWrapper(api_key=self.api_key, model_name=self.model_name)
#         elif self.llm_type == "groq":
#             return GroqLLMWrapper(api_key=self.api_key, model_name=self.model_name)
#         elif self.llm_type == "ollama":
#             if not self.host:
#                 raise ValueError("Host is required for Ollama LLM.")
#             return OllamaLLMWrapper(model_name=self.model_name, host=self.host)
#         else:
#             raise ValueError(f"Unsupported LLM type: {self.llm_type}")
#
#     def getLLMData(self, prompt: str) -> str:
#         llm = self.get_llm_instance()
#         return llm.get_data(prompt)


from kynex.llm.gemini.GeminiLLMWrapper import GeminiLLMWrapper
from kynex.llm.groq.GroqLLMWrapper import GroqLLMWrapper
from kynex.llm.ollama.OllamaLLMWrapper import OllamaLLMWrapper

class LLMConnectorHelper:
    def __init__(self, model_name: str, llm_type: str, api_key: str = None, host: str = None):
        self.model_name = model_name
        self.llm_type = llm_type.lower()
        self.api_key = api_key
        self.host = host

    def get_llm_instance(self):
        print(f"🔍 Selected LLM: {self.llm_type} | Model: {self.model_name}")

        if self.llm_type == "gemini":
            return GeminiLLMWrapper(api_key=self.api_key, model_name=self.model_name)
        elif self.llm_type == "groq":
            return GroqLLMWrapper(api_key=self.api_key, model_name=self.model_name)
        elif self.llm_type == "ollama":
            if not self.host:
                raise ValueError("Host is required for Ollama LLM.")
            return OllamaLLMWrapper(model_name=self.model_name, host=self.host)
        else:
            raise ValueError(f"Unsupported LLM type: {self.llm_type}")

    def getLLMData(self, prompt: str) -> dict:
        llm = self.get_llm_instance()
        result = llm.get_data(prompt)

        if isinstance(result, str):
            result = {
                "response": result,
                "prompt_tokens": None,
                "completion_tokens": None,
                "total_tokens": None
            }

        # 🔹 Auto-print the result (so user doesn't have to call print())
        # print("\n🔹 LLM Response:\n")
        # print(result["response"])
        #
        # if result["total_tokens"] is not None:
        #     print(
        #         f"\n Token Usage → Prompt: {result['prompt_tokens']} | Completion: {result['completion_tokens']} | Total: {result['total_tokens']}")
        # else:
        #     print("\n Token usage information not available.")
        #
        # return result


# import google.generativeai as genai
# from langchain_core.prompts import PromptTemplate
# from kynex.llm.base import LLMBase
#
# class GeminiLLM(LLMBase):
#     def __init__(self, api_key: str, model_name: str = "gemini-1.5-flash"):
#         genai.configure(api_key=api_key)
#         self.model = genai.GenerativeModel(model_name)
#     def get_data(self, prompt: str) -> str:
#         try:
#             template = PromptTemplate.from_template("{prompt}")
#             formatted_prompt = template.format(prompt=prompt)
#             response = self.model.generate_content(formatted_prompt)
#             print(response)
#             return response.text
#         except Exception as e:
#             return f"[Gemini ERROR]: {str(e)}"

#
# import google.generativeai as genai
# from langchain_core.prompts import PromptTemplate
# from kynex.llm.base import LLMBase
#
# class GeminiLLMWrapper(LLMBase):
#     def __init__(self, api_key: str, model_name: str = "gemini-1.5-flash"):
#         genai.configure(api_key=api_key)
#         self.model = genai.GenerativeModel(model_name)
#         print(f"🔹 [GeminiLLM] Initialized with model: {model_name}")
#
#     def get_data(self, prompt: str) -> str:
#         try:
#             print(f"🔹 [GeminiLLM] Generating response for prompt:\n{prompt}")
#             template = PromptTemplate.from_template("{prompt}")
#             formatted_prompt = template.format(prompt=prompt)
#
#             response = self.model.generate_content(formatted_prompt)
#             return "[Gemini] " + response.text
#
#         except Exception as e:
#             return f"[Gemini ERROR]: {str(e)}"
#
# import google.generativeai as genai
# from langchain_core.prompts import PromptTemplate
# from kynex.llm.base import LLMBase
#
# class GeminiLLMWrapper(LLMBase):
#     def __init__(self, api_key: str, model_name: str = "gemini-1.5-flash"):
#         genai.configure(api_key=api_key)
#         self.model = genai.GenerativeModel(model_name)
#         print(f"🔹 [GeminiLLM] Initialized with model: {model_name}")
#
#     def get_data(self, prompt: str) -> dict:
#         try:
#             print(f"🔹 [GeminiLLM] Generating response for prompt:\n{prompt}")
#             template = PromptTemplate.from_template("{prompt}")
#             formatted_prompt = template.format(prompt=prompt)
#
#             response = self.model.generate_content(formatted_prompt)
#
#             # Token usage details from Gemini
#             usage = response.usage_metadata if hasattr(response, "usage_metadata") else {}
#
#             return {
#                 "response": "[Gemini] " + response.text,
#                 "prompt_tokens": usage.get("prompt_token_count", "N/A"),
#                 "completion_tokens": usage.get("candidates_token_count", "N/A"),
#                 "total_tokens": usage.get("total_token_count", "N/A")
#             }
#
#         except Exception as e:
#             return {
#                 "response": f"[Gemini ERROR]: {str(e)}",
#                 "prompt_tokens": None,
#                 "completion_tokens": None,
#                 "total_tokens": None
#             }

import google.generativeai as genai
from langchain_core.prompts import PromptTemplate
from kynex.llm.base import LLMBase

class GeminiLLMWrapper(LLMBase):
    def __init__(self, api_key: str, model_name: str = "gemini-1.5-flash"):
        # Configure Gemini API with your key
        genai.configure(api_key=api_key)
        self.model = genai.GenerativeModel(model_name)
        print(f"🔹 [GeminiLLM] Initialized with model: {model_name}")

    def get_data(self, prompt: str) -> dict:
        try:
            print(f"🔹 [GeminiLLM] Generating response for prompt:\n{prompt}")

            # Optional LangChain formatting
            template = PromptTemplate.from_template("{prompt}")
            formatted_prompt = template.format(prompt=prompt)

            # Generate response from Gemini
            response = self.model.generate_content(formatted_prompt)
            print(response)
            result = response.text.strip()
            # Extract token usage safely using getattr (NOT .get())
            usage = response.usage_metadata

            prompt_tokens = getattr(usage, "prompt_token_count", None)
            completion_tokens = getattr(usage, "candidates_token_count", None)
            total_tokens = getattr(usage, "total_token_count", None)

            return {
                "response": "[Gemini] " + response.text,
                "prompt_tokens": prompt_tokens,
                "completion_tokens": completion_tokens,
                "total_tokens": total_tokens
            }

        except Exception as e:
            return {
                "response": f"[Gemini ERROR]: {str(e)}",
                "prompt_tokens": None,
                "completion_tokens": None,
                "total_tokens": None
            }



# from groq import Groq
# from langchain_core.prompts import PromptTemplate
# from kynex.llm.base import LLMBase
#
# class GroqLLMWrapper(LLMBase):
#     def __init__(self, api_key: str, model_name: str):
#         self.client = Groq(api_key=api_key)
#         self.model_name = model_name
#
#     def get_data(self, prompt: str) -> str:
#         try:
#             # Optional: format prompt using LangChain template
#             template = PromptTemplate.from_template("{prompt}")
#             formatted_prompt = template.format(prompt=prompt)
#
#             response = self.client.chat.completions.create(
#                 model=self.model_name,
#                 messages=[{"role": "user", "content": formatted_prompt}],
#                 temperature=0.7
#             )
#             print(response)
#
#             return response.choices[0].message.content.strip()
#         except Exception as e:
#             return f"[Groq ERROR]: {str(e)}"

from groq import Groq
from langchain_core.prompts import PromptTemplate
from kynex.llm.base import LLMBase

class GroqLLMWrapper(LLMBase):
    def __init__(self, api_key: str, model_name: str):
        self.client = Groq(api_key=api_key)
        self.model_name = model_name

    def get_data(self, prompt: str) -> dict:
        try:
            # Format prompt using LangChain template
            template = PromptTemplate.from_template("{prompt}")
            formatted_prompt = template.format(prompt=prompt)

            response = self.client.chat.completions.create(
                model=self.model_name,
                messages=[{"role": "user", "content": formatted_prompt}],
                temperature=0.7
            )

            content = response.choices[0].message.content.strip()
            usage = response.usage  # Token usage info

            return {
                "response": "[Groq] " + content,
                "prompt_tokens": usage.prompt_tokens,
                "completion_tokens": usage.completion_tokens,
                "total_tokens": usage.total_tokens
            }

        except Exception as e:
            return {
                "response": f"[Groq ERROR]: {str(e)}",
                "prompt_tokens": None,
                "completion_tokens": None,
                "total_tokens": None
            }
